"""
 * FFT Multiplication
 *
 * @author      Moin Khan
 * @copyright   Moin Khan
 *
 * @link https://mo.inkhan.dev
 *
 */
 """

# __init__.py

from .fft_multiplication import multiply, direct_multiply

# Version of the fft-multiplication package
__version__= "0.0.1"
